@@ -117,10 +117,12 @@ typedef intintobjargproc ssizessizeobjargproc;
 #include <unicode/caniter.h>
 #include <unicode/coleitr.h>
 #include <unicode/coll.h>
+#include <unicode/sortkey.h>
 #include <unicode/tblcoll.h>
 #include <unicode/ucsdet.h>
 #include <unicode/uidna.h>
 #include <unicode/translit.h>
+#include <unicode/ulocdata.h>
 #include <unicode/unifunct.h>
 #include <unicode/unimatch.h>
 #include <unicode/unifilt.h>
@@ -130,6 +132,7 @@ typedef intintobjargproc ssizessizeobjargproc;
 #include <unicode/normlzr.h>
 #include <unicode/search.h>
 #include <unicode/stsearch.h>
+#include <unicode/uscript.h>
 
 #define VERSION_HEX(major, minor, patch) \
   (((major) << 24) | ((minor) << 16) | ((patch) << 8))
@@ -160,6 +163,9 @@ typedef intintobjargproc ssizessizeobjargproc;
 #include <typeinfo>
 #endif
 
+#if U_ICU_VERSION_HEX >= VERSION_HEX(50, 0, 0)
+#include <unicode/listformatter.h>
+#endif
 
 #if U_ICU_VERSION_HEX < 0x04060000
 
@@ -180,7 +186,8 @@ enum {
     Calendar_ID,
     Collator_ID,
     UnicodeMatcher_ID,
-    SearchIterator_ID
+    SearchIterator_ID,
+    ListFormatter_ID,
 };
 
 #else
