@@ -92,7 +92,7 @@ class TestCollator(TestCase):
         self.assertEqual(-1, collator.compare(hira_ge, kana_ge))
         hira_ge_key = collator.getSortKey(hira_ge)
         kana_ge_key = collator.getSortKey(kana_ge)
-        self.assertLess(hira_ge_key, kana_ge_key)
+        self.assertTrue(hira_ge_key < kana_ge_key)
 
     def setupCollator(self, collator):
 
@@ -128,7 +128,7 @@ class TestCollator(TestCase):
 
     def testCollatorLoading(self):
 
-        if ICU_VERSION >= '4.6':
+        if ICU_VERSION >= '4.6' and ICU_VERSION <= '52.1':
             collator = self.LoadCollatorFromRules()
             key0 = collator.getSortKey(u'\u3069\u3052\u3056')
             bin = collator.cloneBinary()
